﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace Memento.Common.WaitControl
{
    /// <summary>
    /// 等待提示信息控件
    /// </summary>
    public partial class WaitTip : UserControl
    {
        #region 成员变量
        
        private Window m_Window;
        private UIElementCollection m_RootControls;

        #endregion
        
        #region 构造函数

        /// <summary>
        /// 弹窗等待窗口，注意，此时如果要在异步线程中关闭等待窗口，则调用 Show 方法需要放在异步线程启动之后。
        /// </summary>
        /// <param name="parent">依赖的父窗口</param>
        public WaitTip(Window parent)
        {
            InitializeComponent();

            try
            {
                MouseRightButtonDown += WaitTip_MouseRightButtonDown;

                m_RootControls = null;
                m_Window = new Window();
                m_Window.WindowStyle = WindowStyle.None;
                m_Window.Background = new SolidColorBrush(Colors.Transparent);
                m_Window.AllowsTransparency = true;
                m_Window.Owner = parent;
                m_Window.WindowStartupLocation = WindowStartupLocation.CenterOwner;
                if (parent == null)
                {
                    m_Window.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                }
                m_Window.SizeToContent = SizeToContent.WidthAndHeight;
                m_Window.ShowInTaskbar = false;
                m_Window.Content = this;
            }
            catch (Exception ex)
            {
                throw new Exception("初始化 WaitTip 控件异常，详细信息：\r\n" + ex.Message);
            }
        }

        /// <summary>
        /// 附在已有控件上的等待窗口
        /// </summary>
        /// <param name="rootControls">根控件</param>
        /// <param name="fullGrid">是否占满整个 Grid</param>
        public WaitTip(UIElementCollection rootControls, bool fullGrid = false)
        {
            InitializeComponent();

            try
            {
                MouseRightButtonDown += WaitTip_MouseRightButtonDown;

                m_Window = null;
                m_RootControls = rootControls;
                
                bContainer.BorderThickness = new Thickness(0);
                bContainer.Background = new SolidColorBrush(Colors.Transparent);

                if (fullGrid)
                {
                    Grid.SetColumnSpan(this, 100);
                    Grid.SetRowSpan(this, 100);
                }
            }
            catch (Exception ex)
            {
                throw new Exception("初始化 WaitTip 控件异常，详细信息：\r\n" + ex.Message);
            }
        }

        #endregion

        #region 事件处理

        // 禁用鼠标右键菜单等其他操作
        private void WaitTip_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            e.Handled = true;
        }

        #endregion

        #region 公开方法

        /// <summary>
        /// 显示等待信息
        /// </summary>
        /// <param name="tipContent">等待信息</param>
        public void Show(string tipContent = "请稍候...")
        {
            lbTip.Content = tipContent;
            if (m_Window != null)
            {
                m_Window.ShowDialog();
            }
            else if (m_RootControls != null)
            {
                if (!m_RootControls.Contains(this))
                {
                    m_RootControls.Add(this);
                }
            }
        }

        /// <summary>
        /// 关闭等待信息
        /// </summary>
        public void Close()
        {
            if (m_Window != null)
            {
                m_Window.Dispatcher.BeginInvoke((Action)delegate ()
                {
                    m_Window.Close();
                    m_Window.Content = null;
                });
            }
            else if (m_RootControls != null)
            {
                Dispatcher.BeginInvoke((Action)delegate ()
                {
                    m_RootControls.Remove(this);
                });
            }

            GC.Collect();
            GC.WaitForFullGCComplete();
        }
        
        #endregion
    }
}